@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of Qt Creator.
 **
@@ -9,17 +9,17 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company.  For licensing terms and
-** conditions see http://www.qt.io/terms-conditions.  For further information
-** use the contact form at http://www.qt.io/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPLv3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ****************************************************************************/
 
@@ -28,6 +28,7 @@
 #include <enumeration.h>
 #include <qmlprivategate.h>
 
+#include <QDebug>
 #include <QEvent>
 #include <QQmlContext>
 #include <QQmlError>
@@ -43,6 +44,7 @@
 #include <QQmlParserStatus>
 #include <QTextDocument>
 #include <QLibraryInfo>
+#include <QJSValue>
 
 static bool isSimpleExpression(const QString &expression)
 {
@@ -63,12 +65,7 @@ ObjectNodeInstance::ObjectNodeInstance(QObject *object)
 {
     if (object)
         QObject::connect(m_object.data(), &QObject::destroyed, [=] {
-
-            /*This lambda is save because m_nodeInstanceServer
-            is a smartpointer and object is a dangling pointer anyway.*/
-
-            if (m_nodeInstanceServer)
-                m_nodeInstanceServer->removeInstanceRelationsipForDeletedObject(object);
+            handleObjectDeletion(object);
         });
 }
 
@@ -98,6 +95,14 @@ void ObjectNodeInstance::destroy()
     m_instanceId = -1;
 }
 
+void ObjectNodeInstance::handleObjectDeletion(QObject *object)
+{
+    // We must pass the m_instanceId here, because this instance is no longer
+    // valid, so the wrapper ServerNodeInstance will report -1 for id.
+    if (m_nodeInstanceServer)
+        m_nodeInstanceServer->removeInstanceRelationsipForDeletedObject(object, m_instanceId);
+}
+
 void ObjectNodeInstance::setInstanceId(qint32 id)
 {
     m_instanceId = id;
@@ -125,7 +130,8 @@ void ObjectNodeInstance::initializePropertyWatcher(const ObjectNodeInstance::Poi
     m_signalSpy.setObjectNodeInstance(objectNodeInstance);
 }
 
-void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance)
+void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance,
+                                    InstanceContainer::NodeFlags /*flags*/)
 {
     initializePropertyWatcher(objectNodeInstance);
     QmlPrivateGate::registerNodeInstanceMetaObject(objectNodeInstance->object(), objectNodeInstance->nodeInstanceServer()->engine());
@@ -134,7 +140,7 @@ void ObjectNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNod
 void ObjectNodeInstance::setId(const QString &id)
 {
     if (!m_id.isEmpty() && context()) {
-        context()->engine()->rootContext()->setContextProperty(m_id, 0);
+        context()->engine()->rootContext()->setContextProperty(m_id, nullptr);
     }
 
     if (!id.isEmpty() && context()) {
@@ -174,6 +180,11 @@ bool ObjectNodeInstance::isLayoutable() const
     return false;
 }
 
+bool ObjectNodeInstance::isRenderable() const
+{
+    return false;
+}
+
 bool ObjectNodeInstance::equalGraphicsItem(QGraphicsItem * /*item*/) const
 {
     return false;
@@ -253,7 +264,7 @@ bool ObjectNodeInstance::isAnchoredByChildren() const
 
 QPair<PropertyName, ServerNodeInstance> ObjectNodeInstance::anchor(const PropertyName &/*name*/) const
 {
-    return qMakePair(PropertyName(), ServerNodeInstance());
+    return {PropertyName(), ServerNodeInstance()};
 }
 
 
@@ -262,11 +273,17 @@ static bool isList(const QQmlProperty &property)
     return property.propertyTypeCategory() == QQmlProperty::List;
 }
 
+static bool isQJSValue(const QQmlProperty &property)
+{
+    return property.isValid() && !strcmp(property.propertyTypeName(), "QJSValue");
+}
+
 static bool isObject(const QQmlProperty &property)
 {
-    return (property.propertyTypeCategory() == QQmlProperty::Object) ||
-            //QVariant can also store QObjects. Lets trust our model.
-           (QLatin1String(property.propertyTypeName()) == QLatin1String("QVariant"));
+    /* QVariant and QJSValue can also store QObjects. Lets trust our model. */
+    return property.isValid() && (property.propertyTypeCategory() == QQmlProperty::Object
+            || !strcmp(property.propertyTypeName(), "QVariant")
+            || isQJSValue(property));
 }
 
 static QVariant objectToVariant(QObject *object)
@@ -301,7 +318,7 @@ static void removeObjectFromList(const QQmlProperty &property, QObject *objectTo
 
 void ObjectNodeInstance::removeFromOldProperty(QObject *object, QObject *oldParent, const PropertyName &oldParentProperty)
 {
-    QQmlProperty property(oldParent, oldParentProperty, context());
+    QQmlProperty property(oldParent, QString::fromUtf8(oldParentProperty), context());
 
     if (!property.isValid())
         return;
@@ -315,12 +332,12 @@ void ObjectNodeInstance::removeFromOldProperty(QObject *object, QObject *oldPare
     }
 
     if (object && object->parent())
-        object->setParent(0);
+        object->setParent(nullptr);
 }
 
 void ObjectNodeInstance::addToNewProperty(QObject *object, QObject *newParent, const PropertyName &newParentProperty)
 {
-    QQmlProperty property(newParent, newParentProperty, context());
+    QQmlProperty property(newParent, QString::fromUtf8(newParentProperty), context());
 
     if (object)
         object->setParent(newParent);
@@ -335,7 +352,14 @@ void ObjectNodeInstance::addToNewProperty(QObject *object, QObject *newParent, c
 
         list.append(object);
     } else if (isObject(property)) {
-        property.write(objectToVariant(object));
+        if (isQJSValue(property)) /* In this case we have to explcitly generate and convert a QJSValue */
+            property.write(QVariant::fromValue(engine()->newQObject(object)));
+        else
+            property.write(objectToVariant(object));
+
+        if (QQuickItem *item = qobject_cast<QQuickItem *>(object))
+            if (QQuickItem *newParentItem = qobject_cast<QQuickItem *>(newParent))
+                item->setParentItem(newParentItem);
     }
 
     Q_ASSERT(objectToVariant(object).isValid());
@@ -376,6 +400,41 @@ PropertyNameList ObjectNodeInstance::ignoredProperties() const
     return PropertyNameList();
 }
 
+void ObjectNodeInstance::setHiddenInEditor(bool b)
+{
+    m_isHiddenInEditor = b;
+}
+
+bool ObjectNodeInstance::isHiddenInEditor() const
+{
+    return m_isHiddenInEditor;
+}
+
+void ObjectNodeInstance::setLockedInEditor(bool b)
+{
+    m_isLockedInEditor = b;
+}
+
+bool ObjectNodeInstance::isLockedInEditor() const
+{
+    return m_isLockedInEditor;
+}
+
+bool ObjectNodeInstance::isComponentWrap() const
+{
+    return m_isComponentWrap;
+}
+
+void ObjectNodeInstance::setComponentWrap(bool wrap)
+{
+    m_isComponentWrap = wrap;
+}
+
+void ObjectNodeInstance::setModifiedFlag(bool b)
+{
+    m_isModified = b;
+}
+
 QVariant ObjectNodeInstance::convertEnumToValue(const QVariant &value, const PropertyName &name)
 {
     Q_ASSERT(value.canConvert<Enumeration>());
@@ -390,7 +449,7 @@ QVariant ObjectNodeInstance::convertEnumToValue(const QVariant &value, const Pro
         QQmlExpression expression(context(), object(), enumeration.toString());
         adjustedValue =  expression.evaluate();
         if (expression.hasError())
-            qDebug() << "Enumeration can not be evaluated:" << object() << name << enumeration;
+            qDebug() << "Enumeration cannot be evaluated:" << object() << name << enumeration;
     }
     return adjustedValue;
 }
@@ -400,7 +459,10 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (ignoredProperties().contains(name))
         return;
 
-    QQmlProperty property(object(), name, context());
+    if (m_isModified)
+        return;
+
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
 
     if (!property.isValid())
         return;
@@ -416,7 +478,7 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (oldValue.type() == QVariant::Url) {
         QUrl url = oldValue.toUrl();
         QString path = url.toLocalFile();
-        if (QFileInfo(path).exists() && nodeInstanceServer() && !path.isEmpty())
+        if (QFileInfo::exists(path) && nodeInstanceServer() && !path.isEmpty())
             nodeInstanceServer()->removeFilePropertyFromFileSystemWatcher(object(), name, path);
     }
 
@@ -433,7 +495,7 @@ void ObjectNodeInstance::setPropertyVariant(const PropertyName &name, const QVar
     if (newValue.type() == QVariant::Url) {
         QUrl url = newValue.toUrl();
         QString path = url.toLocalFile();
-        if (QFileInfo(path).exists() && nodeInstanceServer() && !path.isEmpty())
+        if (QFileInfo::exists(path) && nodeInstanceServer() && !path.isEmpty())
             nodeInstanceServer()->addFilePropertyToFileSystemWatcher(object(), name, path);
     }
 }
@@ -482,7 +544,7 @@ void ObjectNodeInstance::resetProperty(const PropertyName &name)
 
 void ObjectNodeInstance::refreshProperty(const PropertyName &name)
 {
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
 
     if (!property.isValid())
         return;
@@ -523,7 +585,7 @@ QVariant ObjectNodeInstance::property(const PropertyName &name) const
     if (QmlPrivateGate::isPropertyBlackListed(name))
         return QVariant();
 
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
     if (property.property().isEnumType()) {
         QVariant value = property.read();
         return property.property().enumerator().valueToKey(value.toInt());
@@ -543,11 +605,30 @@ QVariant ObjectNodeInstance::property(const PropertyName &name) const
     return property.read();
 }
 
+void ObjectNodeInstance::ensureVector3DDotProperties(PropertyNameList &list) const
+{
+    const PropertyNameList properties = { "rotation", "scale", "pivot" };
+    for (const auto &property : properties) {
+        if (list.contains(property) && instanceType(property) == "QVector3D") {
+            const PropertyNameList dotProperties = { "x", "y", "z" };
+            for (const auto &dotProperty : dotProperties) {
+                const PropertyName dotPropertyName = property + "." + dotProperty;
+                if (!list.contains(dotPropertyName))
+                    list.append(dotPropertyName);
+            }
+        }
+    }
+}
+
 PropertyNameList ObjectNodeInstance::propertyNames() const
 {
+    PropertyNameList list;
     if (isValid())
-        return QmlPrivateGate::allPropertyNames(object());
-    return PropertyNameList();
+        list = QmlPrivateGate::allPropertyNames(object());
+
+    ensureVector3DDotProperties(list);
+
+    return list;
 }
 
 QString ObjectNodeInstance::instanceType(const PropertyName &name) const
@@ -555,10 +636,10 @@ QString ObjectNodeInstance::instanceType(const PropertyName &name) const
     if (QmlPrivateGate::isPropertyBlackListed(name))
         return QLatin1String("undefined");
 
-    QQmlProperty property(object(), name, context());
+    QQmlProperty property(object(), QString::fromUtf8(name), context());
     if (!property.isValid())
         return QLatin1String("undefined");
-    return property.propertyTypeName();
+    return QString::fromUtf8(property.propertyTypeName());
 }
 
 QList<ServerNodeInstance> ObjectNodeInstance::childItems() const
@@ -601,7 +682,61 @@ ObjectNodeInstance::Pointer ObjectNodeInstance::create(QObject *object)
 
 QObject *ObjectNodeInstance::createPrimitive(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context)
 {
-    return QmlPrivateGate::createPrimitive(typeName, majorNumber, minorNumber, context);
+    QString polishTypeName = typeName;
+    if (typeName == "QtQuick.Controls/Popup" || typeName == "QtQuick.Controls/Drawer"
+        || typeName == "QtQuick.Controls/Dialog" || typeName == "QtQuick.Controls/Menu"
+        || typeName == "QtQuick.Controls/ToolTip"
+        || typeName == "QtQuick.Controls/ApplicationWindow")
+        polishTypeName = "QtQuick/Item";
+
+    const QHash<QString, QString> mockHash = {{"QtQuick.Controls/SwipeView","qrc:/qtquickplugin/mockfiles/SwipeView.qml"},
+                                             {"QtQuick.Dialogs/Dialog","qrc:/qtquickplugin/mockfiles/Dialog.qml"}};
+
+    QObject *object = nullptr;
+
+    if (mockHash.contains(typeName))
+        object = QmlPrivateGate::createComponent(mockHash.value(typeName), context);
+    else if (majorNumber != -1 && minorNumber != -1)
+        object = QmlPrivateGate::createPrimitive(polishTypeName, majorNumber, minorNumber, context);
+    else
+        object = QmlPrivateGate::createPrimitive(polishTypeName, 0, 0, context);
+
+    /* Let's try to create the primitive from source, since with incomplete meta info this might be a pure
+     * QML type. This is the case for example if a C++ type is mocked up with a QML file.
+     */
+
+    if (!object && majorNumber == 6 && minorNumber == 0) {
+        object = QmlPrivateGate::createPrimitive(polishTypeName, 2, 15, context);
+        if (!object)
+            object = QmlPrivateGate::createPrimitive(polishTypeName, 1, 0, context);
+    }
+
+    if (!object)
+        object = createPrimitiveFromSource(polishTypeName, majorNumber, minorNumber, context);
+
+    return object;
+}
+
+QObject *ObjectNodeInstance::createPrimitiveFromSource(const QString &typeName, int majorNumber, int minorNumber, QQmlContext *context)
+{
+    if (typeName.isEmpty())
+        return nullptr;
+
+    QStringList parts = typeName.split("/");
+    const QString unqualifiedTypeName = parts.last();
+    parts.removeLast();
+
+    if (parts.isEmpty())
+        return nullptr;
+
+    QString importString = parts.join(".");
+    if (majorNumber >= 0)
+        importString = importString + " " + QString::number(majorNumber) + "."
+                       + QString::number(minorNumber);
+    if (importString == "QtQuick 1.0") /* Workaround for implicit QQml import */
+        importString = "QtQuick 2.0";
+    QString source = "import " + importString + "\n" + unqualifiedTypeName + " {\n" + "}\n";
+    return createCustomParserObject(source, "", context);
 }
 
 QObject *ObjectNodeInstance::createComponentWrap(const QString &nodeSource, const QByteArray &importCode, QQmlContext *context)
@@ -612,6 +747,10 @@ QObject *ObjectNodeInstance::createComponentWrap(const QString &nodeSource, cons
     QQmlComponent *component = new QQmlComponent(context->engine());
 
     QByteArray data(nodeSource.toUtf8());
+    if (data.isEmpty()) {
+        // Add a fake root element as an empty component is not valid and crashes in some cases
+        data.append("QtObject{}");
+    }
     data.prepend(importCode);
     component->setData(data, context->baseUrl().resolved(QUrl("createComponent.qml")));
     QObject *object = component;
@@ -638,16 +777,16 @@ static inline QString fixComponentPathForIncompatibleQt(const QString &component
     if (componentPath.contains(importString)) {
         int index = componentPath.indexOf(importString) + 8;
         const QString relativeImportPath = componentPath.right(componentPath.length() - index);
-        QString fixedComponentPath = QLibraryInfo::location(QLibraryInfo::ImportsPath) + relativeImportPath;
+        QString fixedComponentPath = QLibraryInfo::location(QLibraryInfo::Qml2ImportsPath) + relativeImportPath;
         fixedComponentPath.replace(QLatin1Char('\\'), QLatin1Char('/'));
-        if (QFileInfo(fixedComponentPath).exists())
+        if (QFileInfo::exists(fixedComponentPath))
             return fixedComponentPath;
         QString fixedPath = QFileInfo(fixedComponentPath).path();
         if (fixedPath.endsWith(QLatin1String(".1.0"))) {
         //plugin directories might contain the version number
             fixedPath.chop(4);
             fixedPath += QLatin1Char('/') + QFileInfo(componentPath).fileName();
-            if (QFileInfo(fixedPath).exists())
+            if (QFileInfo::exists(fixedPath))
                 return fixedPath;
         }
     }
@@ -662,10 +801,14 @@ QObject *ObjectNodeInstance::createComponent(const QString &componentPath, QQmlC
     Q_UNUSED(disableComponentComplete)
 
     QQmlComponent component(context->engine(), fixComponentPathForIncompatibleQt(componentPath));
-    QObject *object = component.beginCreate(context);
 
-    QmlPrivateGate::tweakObjects(object);
-    component.completeCreate();
+    QObject *object = nullptr;
+    if (!component.isError()) {
+        object = component.beginCreate(context);
+        QmlPrivateGate::tweakObjects(object);
+        component.completeCreate();
+        QQmlEngine::setObjectOwnership(object, QQmlEngine::CppOwnership);
+    }
 
     if (component.isError()) {
         qDebug() << componentPath;
@@ -673,7 +816,8 @@ QObject *ObjectNodeInstance::createComponent(const QString &componentPath, QQmlC
             qWarning() << error;
     }
 
-    QQmlEngine::setObjectOwnership(object, QQmlEngine::CppOwnership);
+    if (object)
+        object->setProperty("__designer_url__", QUrl::fromLocalFile(componentPath));
 
     return object;
 }
@@ -683,7 +827,9 @@ QObject *ObjectNodeInstance::createComponent(const QUrl &componentUrl, QQmlConte
     return QmlPrivateGate::createComponent(componentUrl, context);
 }
 
-QObject *ObjectNodeInstance::createCustomParserObject(const QString &nodeSource, const QByteArray &importCode, QQmlContext *context)
+QObject *ObjectNodeInstance::createCustomParserObject(const QString &nodeSource,
+                                                      const QByteArray &importCode,
+                                                      QQmlContext *context)
 {
     QmlPrivateGate::ComponentCompleteDisabler disableComponentComplete;
     Q_UNUSED(disableComponentComplete)
@@ -694,9 +840,11 @@ QObject *ObjectNodeInstance::createCustomParserObject(const QString &nodeSource,
     data.prepend(importCode);
     component.setData(data, context->baseUrl().resolved(QUrl("createCustomParserObject.qml")));
     QObject *object = component.beginCreate(context);
-    QmlPrivateGate::tweakObjects(object);
-    component.completeCreate();
-    QQmlEngine::setObjectOwnership(object, QQmlEngine::CppOwnership);
+    if (object) {
+        QmlPrivateGate::tweakObjects(object);
+        component.completeCreate();
+        QQmlEngine::setObjectOwnership(object, QQmlEngine::CppOwnership);
+    }
 
     if (component.isError()) {
         qWarning() << "Error in:" << Q_FUNC_INFO << component.url().toString();
@@ -711,12 +859,12 @@ QObject *ObjectNodeInstance::object() const
 {
         if (!m_object.isNull() && !QmlPrivateGate::objectWasDeleted(m_object.data()))
             return m_object.data();
-        return 0;
+        return nullptr;
 }
 
 QQuickItem *ObjectNodeInstance::contentItem() const
 {
-    return 0;
+    return nullptr;
 }
 
 bool ObjectNodeInstance::hasContent() const
@@ -758,7 +906,7 @@ QQmlContext *ObjectNodeInstance::context() const
         return nodeInstanceServer()->context();
 
     qWarning() << "Error: No NodeInstanceServer";
-    return 0;
+    return nullptr;
 }
 
 QQmlEngine *ObjectNodeInstance::engine() const
@@ -778,6 +926,11 @@ void ObjectNodeInstance::deactivateState()
 {
 }
 
+QStringList ObjectNodeInstance::allStates() const
+{
+    return {};
+}
+
 void ObjectNodeInstance::populateResetHashes()
 {
     QmlPrivateGate::registerCustomData(object());
@@ -803,10 +956,15 @@ QImage ObjectNodeInstance::renderPreviewImage(const QSize & /*previewImageSize*/
     return QImage();
 }
 
+QSharedPointer<QQuickItemGrabResult> ObjectNodeInstance::createGrabResult() const
+{
+    return {};
+}
+
 QObject *ObjectNodeInstance::parent() const
 {
     if (!object())
-        return 0;
+        return nullptr;
 
     return object()->parent();
 }
@@ -814,14 +972,8 @@ QObject *ObjectNodeInstance::parent() const
 QObject *ObjectNodeInstance::parentObject(QObject *object)
 {
     QQuickItem *quickItem = qobject_cast<QQuickItem*>(object);
-    if (quickItem && quickItem->parentItem()) {
-
-        //QQuickRootItem is used by Window and we want to return the Window as parent
-        if (strcmp(quickItem->metaObject()->className(), "QQuickRootItem"))
-            return object->parent();
-
+    if (quickItem && quickItem->parentItem())
         return quickItem->parentItem();
-    }
 
     return object->parent();
 }
