@@ -28,6 +28,9 @@
 
 #include <qmlprivategate.h>
 
+#include <QtQuick/private/qquickitem_p.h>
+#include <QtQuick/private/qquickshadereffectsource_p.h>
+
 #include <QQmlProperty>
 #include <QQmlExpression>
 #include <QQuickView>
@@ -59,8 +62,15 @@ QuickItemNodeInstance::QuickItemNodeInstance(QQuickItem *item)
 
 QuickItemNodeInstance::~QuickItemNodeInstance()
 {
-    if (quickItem() && checkIfRefFromEffect(instanceId()))
-        designerSupport()->derefFromEffectItem(quickItem());
+}
+
+void QuickItemNodeInstance::handleObjectDeletion(QObject *object)
+{
+    auto item = qobject_cast<QQuickItem *>(object);
+    if (item && checkIfRefFromEffect(instanceId()))
+        designerSupport()->derefFromEffectItem(item);
+
+    ObjectNodeInstance::handleObjectDeletion(object);
 }
 
 static bool isContentItem(QQuickItem *item, NodeInstanceServer *nodeInstanceServer)
@@ -164,10 +174,15 @@ void QuickItemNodeInstance::enableUnifiedRenderPath(bool unifiedRenderPath)
 
 bool QuickItemNodeInstance::checkIfRefFromEffect(qint32 id)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (s_unifiedRenderPath)
         return false;
 
     return (s_createEffectItem || id == 0);
+#else
+    Q_UNUSED(id)
+    return false;
+#endif
 }
 
 void QuickItemNodeInstance::initialize(const ObjectNodeInstance::Pointer &objectNodeInstance,
@@ -260,9 +275,13 @@ QStringList QuickItemNodeInstance::allStates() const
 
 void QuickItemNodeInstance::updateDirtyNode(QQuickItem *item)
 {
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
     if (s_unifiedRenderPath)
         return;
     DesignerSupport::updateDirtyNode(item);
+#else
+    Q_UNUSED(item)
+#endif
 }
 
 bool QuickItemNodeInstance::unifiedRenderPath()
@@ -270,6 +289,36 @@ bool QuickItemNodeInstance::unifiedRenderPath()
     return s_unifiedRenderPath;
 }
 
+bool QuickItemNodeInstance::unifiedRenderPathOrQt6()
+{
+#if QT_VERSION >= QT_VERSION_CHECK(6, 0, 0)
+    return true;
+#else
+    return s_unifiedRenderPath;
+#endif
+}
+
+void QuickItemNodeInstance::setHiddenInEditor(bool hide)
+{
+    ObjectNodeInstance::setHiddenInEditor(hide);
+    if (s_unifiedRenderPath && !nodeInstanceServer()->isInformationServer()) {
+        QQmlProperty property(object(), "visible", context());
+
+        if (!property.isValid())
+            return;
+
+        bool visible = property.read().toBool();
+
+        if (hide && visible) {
+            setPropertyVariant("visible", false);
+            m_hidden = true;
+        } else if (!hide && !visible && m_hidden) {
+            setPropertyVariant("visible", true);
+            m_hidden = false;
+        }
+    }
+}
+
 QRectF QuickItemNodeInstance::contentItemBoundingBox() const
 {
     if (contentItem()) {
@@ -280,13 +329,37 @@ QRectF QuickItemNodeInstance::contentItemBoundingBox() const
     return QRectF();
 }
 
+static bool layerEnabledAndEffect(QQuickItem *item)
+{
+    QQuickItemPrivate *pItem = QQuickItemPrivate::get(item);
+
+    if (pItem && pItem->layer() && pItem->layer()->enabled() && pItem->layer()->effect())
+        return true;
+
+    return false;
+}
+
 QRectF QuickItemNodeInstance::boundingRect() const
 {
     if (quickItem()) {
         if (quickItem()->clip()) {
             return quickItem()->boundingRect();
+        } else if (layerEnabledAndEffect(quickItem())) {
+            return ServerNodeInstance::effectAdjustedBoundingRect(quickItem());
         } else {
-            return boundingRectWithStepChilds(quickItem());
+            QSize maximumSize(4000, 4000);
+            auto isValidSize = [maximumSize] (const QRectF& rect) {
+                QSize size = rect.size().toSize();
+                return size.width() * size.height() <= maximumSize.width() * maximumSize.height();
+            };
+
+            QRectF rect = boundingRectWithStepChilds(quickItem());
+            if (isValidSize(rect))
+                return rect;
+            else if (rect = quickItem()->boundingRect(); isValidSize(rect))
+                return rect;
+            else
+                return QRectF(QPointF(0.0, 0.0), maximumSize);
         }
     }
 
@@ -408,16 +481,13 @@ QImage QuickItemNodeInstance::renderImage() const
     updateDirtyNodesRecursive(quickItem());
 
     QRectF renderBoundingRect = boundingRect();
-
-    QSize size = renderBoundingRect.size().toSize();
-    static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
-    if (size.width() * size.height() > 4000 * 4000)
-        size = QSize(0,0);
-    size *= devicePixelRatio;
-
     QImage renderImage;
 
 #if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    QSize size = renderBoundingRect.size().toSize();
+    static double devicePixelRatio = qgetenv("FORMEDITOR_DEVICE_PIXEL_RATIO").toDouble();
+    size *= devicePixelRatio;
+
     if (s_unifiedRenderPath) {
         renderImage = nodeInstanceServer()->quickWindow()->grabWindow();
     } else {
@@ -431,8 +501,13 @@ QImage QuickItemNodeInstance::renderImage() const
     }
     renderImage.setDevicePixelRatio(devicePixelRatio);
 #else
-    renderImage = nodeInstanceServer()->grabWindow();
-    renderImage = renderImage.copy(renderBoundingRect.toRect());
+    if (s_unifiedRenderPath) {
+        renderImage = nodeInstanceServer()->grabWindow();
+        renderImage = renderImage.copy(renderBoundingRect.toRect());
+    } else {
+        renderImage = nodeInstanceServer()->grabItem(quickItem());
+    }
+
     /* When grabbing an offscren window the device pixel ratio is 1 */
     renderImage.setDevicePixelRatio(1);
 #endif
@@ -496,6 +571,11 @@ bool QuickItemNodeInstance::isQuickItem() const
     return true;
 }
 
+bool QuickItemNodeInstance::isRenderable() const
+{
+    return quickItem() && (!s_unifiedRenderPath || isRootNodeInstance());
+}
+
 QList<ServerNodeInstance> QuickItemNodeInstance::stateInstances() const
 {
     QList<ServerNodeInstance> instanceList;
@@ -559,11 +639,51 @@ void QuickItemNodeInstance::updateAllDirtyNodesRecursive(QQuickItem *parentItem)
     updateDirtyNode(parentItem);
 }
 
+void QuickItemNodeInstance::setAllNodesDirtyRecursive(QQuickItem *parentItem) const
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    Q_UNUSED(parentItem)
+#else
+    const QList<QQuickItem *> children = parentItem->childItems();
+    for (QQuickItem *childItem : children)
+        setAllNodesDirtyRecursive(childItem);
+    DesignerSupport::addDirty(parentItem, QQuickDesignerSupport::Content);
+#endif
+}
+
 static inline bool isRectangleSane(const QRectF &rect)
 {
     return rect.isValid() && (rect.width() < 10000) && (rect.height() < 10000);
 }
 
+static bool isEffectItem(QQuickItem *item)
+{
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
+    Q_UNUSED(item)
+    return false;
+#else
+    if (qobject_cast<QQuickShaderEffectSource *>(item))
+        return true;
+
+    const auto propName = "source";
+
+    QQmlProperty prop(item, QString::fromLatin1(propName));
+    if (!prop.isValid())
+        return false;
+
+    QQuickShaderEffectSource *source = prop.read().value<QQuickShaderEffectSource *>();
+
+    if (source && source->sourceItem()) {
+        QQuickItemPrivate *pItem = QQuickItemPrivate::get(source->sourceItem());
+
+        if (pItem && pItem->layer() && pItem->layer()->enabled() && pItem->layer()->effect())
+            return true;
+    }
+
+    return false;
+#endif
+}
+
 QRectF QuickItemNodeInstance::boundingRectWithStepChilds(QQuickItem *parentItem) const
 {
     QRectF boundingRect = parentItem->boundingRect();
@@ -571,8 +691,9 @@ QRectF QuickItemNodeInstance::boundingRectWithStepChilds(QQuickItem *parentItem)
     boundingRect = boundingRect.united(QRectF(QPointF(0, 0), size()));
 
     for (QQuickItem *childItem : parentItem->childItems()) {
-        if (!nodeInstanceServer()->hasInstanceForObject(childItem)) {
-            QRectF transformedRect = childItem->mapRectToItem(parentItem, boundingRectWithStepChilds(childItem));
+        if (!nodeInstanceServer()->hasInstanceForObject(childItem) && !isEffectItem(childItem)) {
+            QRectF transformedRect = childItem->mapRectToItem(parentItem,
+                                                              boundingRectWithStepChilds(childItem));
             if (isRectangleSane(transformedRect))
                 boundingRect = boundingRect.united(transformedRect);
         }
@@ -673,8 +794,13 @@ void QuickItemNodeInstance::reparent(const ObjectNodeInstance::Pointer &oldParen
         setMovable(true);
     }
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::reparent(oldParentInstance, oldParentProperty, newParentInstance, newParentProperty);
 
+    if (!newParentInstance)
+        quickItem()->setParentItem(nullptr);
+
     if (instanceIsValidLayoutable(newParentInstance, newParentProperty)) {
         setInLayoutable(true);
         setMovable(false);
@@ -731,6 +857,11 @@ void QuickItemNodeInstance::setPropertyVariant(const PropertyName &name, const Q
     if (name == "y")
         m_y = value.toDouble();
 
+    if (name == "layer.enabled" || name == "layer.effect")
+        setAllNodesDirtyRecursive(quickItem());
+
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::setPropertyVariant(name, value);
 
     refresh();
@@ -750,6 +881,8 @@ void QuickItemNodeInstance::setPropertyBinding(const PropertyName &name, const Q
     if (name.startsWith("anchors.") && isRootNodeInstance())
         return;
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::setPropertyBinding(name, expression);
 
     refresh();
@@ -800,6 +933,9 @@ void QuickItemNodeInstance::resetProperty(const PropertyName &name)
     if (name == "y")
         m_y = 0.0;
 
+    if (name == "layer.enabled" || name == "layer.effect")
+        setAllNodesDirtyRecursive(quickItem());
+
     DesignerSupport::resetAnchor(quickItem(), QString::fromUtf8(name));
 
     if (name == "anchors.fill") {
@@ -824,6 +960,8 @@ void QuickItemNodeInstance::resetProperty(const PropertyName &name)
         resetVertical();
     }
 
+    markRepeaterParentDirty();
+
     ObjectNodeInstance::resetProperty(name);
 
     if (isInLayoutable())
@@ -900,6 +1038,35 @@ QQuickItem *QuickItemNodeInstance::quickItem() const
     return static_cast<QQuickItem*>(object());
 }
 
+void QuickItemNodeInstance::markRepeaterParentDirty() const
+{
+    const qint32 id = instanceId();
+    if (id <= 0 && !isValid())
+        return;
+
+    QQuickItem *item = quickItem();
+    if (!item)
+        return;
+
+    QQuickItem *parentItem = item->parentItem();
+    if (!parentItem)
+        return;
+
+    // If a Repeater instance was changed in any way, the parent must be marked dirty to rerender it
+    const QByteArray type("QQuickRepeater");
+    if (ServerNodeInstance::isSubclassOf(item, type))
+        DesignerSupport::addDirty(parentItem, QQuickDesignerSupport::Content);
+
+    // Repeater's parent must also be dirtied when a child of a repeater was changed
+    if (ServerNodeInstance::isSubclassOf(parentItem, type)) {
+        QQuickItem *parentsParent = parentItem->parentItem();
+        if (parentsParent)
+            DesignerSupport::addDirty(parentsParent, QQuickDesignerSupport::Content);
+    }
+}
+
+
+
 } // namespace Internal
 } // namespace QmlDesigner
 
